// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using System.Linq;
using System.Text;

using Smdn.IO;

namespace Smdn.Formats.Mime.Formatting {
 internal static class Formatter {
    public static void Format(MimeMessageBuilder builder, Stream stream)
    {
      var multipart = 0 < builder.SubParts.Count;

      if (multipart && builder.Boundary == null)
        throw new NullReferenceException("message is multipart, but boudnary was null");

      if (builder.GetHeaders(MimeHeaderNames.MimeVersion).Count() == 0)
        builder.Headers.Insert(0, new MimeHeaderBuilder(MimeHeaderNames.MimeVersion, "1.0"));

      if (builder.GetHeaders(MimeHeaderNames.ContentType).Count() == 0)
        builder.Headers.Insert(1, MimeHeaderBuilder.CreateContentType(builder.Format,
                                                                      builder.MimeType ?? (multipart ? MimeType.MultipartMixed : MimeType.TextPlain),
                                                                      builder.Charset,
                                                                      multipart ? builder.Boundary : null));

      if (builder.GetHeaders(MimeHeaderNames.ContentTransferEncoding).Count() == 0)
        builder.Headers.Insert(1, MimeHeaderBuilder.CreateContentTransferEncoding(builder.Format, builder.TransferEncoding));

      var textWriter = new StreamWriter(stream, Charsets.ISO8859_1);
      var eol = (builder.Format ?? MimeFormat.Unspecified).GetEOLString();

      textWriter.NewLine = eol;

      foreach (var header in builder.Headers) {
        textWriter.WriteLine(header.ToString());
      }

      if (multipart) {
        var delimiter = string.Concat(eol, "--", builder.Boundary);

        foreach (var part in builder.SubParts) {
          textWriter.WriteLine(delimiter);
          textWriter.Flush();

          Format(part, stream);
        }

        textWriter.WriteLine(string.Concat(delimiter, "--"));
        textWriter.Flush();
      }
      else {
        textWriter.WriteLine();
        textWriter.Flush();

        if (builder.Content != null) {
          builder.Content.Position = 0L;
          builder.Content.CopyTo(stream);

          stream.Flush();
        }
      }
    }
  }
}