//
// Author:
//       smdn <smdn@smdn.jp>
//
// Copyright (c) 2008-2014 smdn
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;

namespace Smdn.Formats.Mime {
  public abstract class MailBase {
    public MimeMessage BaseMessage {
      get { CheckDisposed(); return baseMessage; }
    }

    protected MailBase(MimeMessage baseMessage)
    {
      if (baseMessage == null)
        throw new ArgumentNullException("baseMessage");

      this.baseMessage = baseMessage;
    }

    protected void DisposeCore()
    {
      if (baseMessage != null) {
        baseMessage.Dispose();
        baseMessage = null;
      }
    }

    protected void CheckDisposed()
    {
      if (baseMessage == null)
        throw new ObjectDisposedException(GetType().FullName);
    }

    public void ReadContent(Action<Stream> read)
    {
      CheckDisposed();

      baseMessage.ReadContent(read);
    }

    public void ReadContent(bool decode, Action<Stream> read)
    {
      CheckDisposed();

      baseMessage.ReadContent(decode, read);
    }

    public void ReadContent(Action<StreamReader> read)
    {
      CheckDisposed();

      baseMessage.ReadContent(read);
    }

    public void ReadContent(Action<BinaryReader> read)
    {
      CheckDisposed();

      baseMessage.ReadContent(read);
    }

    public string ReadContentAsText()
    {
      CheckDisposed();

      return baseMessage.ReadContentAsText();
    }

    public byte[] ReadContentAsBinary()
    {
      CheckDisposed();

      return baseMessage.ReadContentAsBinary();
    }

    public void CopyContentTo(Stream stream)
    {
      CheckDisposed();

      baseMessage.CopyContentTo(stream);
    }

    public void CopyContentTo(Stream stream, bool decode)
    {
      CheckDisposed();

      baseMessage.CopyContentTo(stream, decode);
    }

    public void SaveContent(string path)
    {
      CheckDisposed();

      baseMessage.SaveContent(path);
    }

    public void SaveContent(string path, bool decode)
    {
      CheckDisposed();

      baseMessage.SaveContent(path, decode);
    }

    internal void SaveContent(FileInfo file, bool decode)
    {
      CheckDisposed();

      baseMessage.SaveContent(file, decode);
    }

    public override string ToString()
    {
      CheckDisposed();

      return baseMessage.ToString();
    }

    private MimeMessage baseMessage;
  }
}

