// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace Smdn.Formats.Mime {
  public class MimeHeaderCollection : ReadOnlyCollection<MimeHeader> {
    public MimeHeader this [string name] {
      get{ return dictionary[name]; }
    }

    internal MimeHeaderCollection(IList<MimeHeader> headers)
      : base(headers)
    {
      dictionary = new Dictionary<string, MimeHeader>(headers.Count, StringComparer.OrdinalIgnoreCase);

      foreach (var header in headers) {
        if (!dictionary.ContainsKey(header.Name))
          dictionary[header.Name] = header;
      }
    }

    public bool Contains(string name)
    {
      return dictionary.ContainsKey(name);
    }

    public bool TryGetHeader(string name, out MimeHeader header)
    {
      return dictionary.TryGetValue(name, out header);
    }

    public IEnumerable<MimeHeader> GetHeaders(string name)
    {
      foreach (var header in this) {
        if (header.IsNameEquals(name))
          yield return header;
      }
    }

    private Dictionary<string, MimeHeader> dictionary;
  }
}
