//
// Author:
//       smdn <smdn@smdn.jp>
//
// Copyright (c) 2008-2014 smdn
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;

namespace Smdn.Formats.Mime {
  internal static class MimeTypeUtils {
    public static string FindExtensionFromMimeType(MimeType mimeType)
    {
      string extension;

      if (mimeTypeExtensionMap.TryGetValue(mimeType.ToString(), out extension))
        return extension;
      else
        return MimeType.FindExtensionsByMimeType(mimeType).FirstOrDefault();
    }

    private static readonly Dictionary<string, string> mimeTypeExtensionMap = new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase) {
      // text
      {"text/csv",                          ".csv"},
      {"text/plain",                        ".txt"},
      {"text/html",                         ".html"},
      {"text/xml",                          ".xml"},
      //{"text/", "."},

      // message
      {"message/html",                      ".html"},
      {"message/rfc822",                    ".eml"},
      //{"message/", "."},

      // image
      {"image/bmp",                         ".bmp"},
      {"image/gif",                         ".gif"},
      {"image/jpeg",                        ".jpg"},
      {"image/png",                         ".png"},
      {"image/svg+xml",                     ".svg"},
      {"image/tiff",                        ".tiff"},
      {"image/vnd.microsoft.icon",          ".ico"},
      {"image/x-bmp",                       ".bmp"},
      {"image/x-icon",                      ".ico"},
      {"image/x-ms-bmp",                    ".bmp"},
      {"image/x-png",                       ".png"},
      {"image/x-portable-anymap",           ".pnm"},
      {"image/x-portable-bitmap",           ".pbm"},
      {"image/x-portable-graymap",          ".pgm"},
      {"image/x-portable-pixmap",           ".ppm"},
      {"image/x-tiff",                      ".tiff"},
      //{"image/", "."},

      // audio
      {"audio/mp3",                         ".mp3"},
      {"audio/mp4",                         ".mp4"},
      {"audio/mpeg",                        ".mp3"},
      {"audio/mpegurl",                     ".m3u"},
      {"audio/mpg",                         ".mp3"},
      {"audio/vnd.wave",                    ".wav"},
      {"audio/wav",                         ".wav"},
      {"audio/webm",                        ".webm"},
      {"audio/x-mp3",                       ".mp3"},
      {"audio/x-mpeg",                      ".mp3"},
      {"audio/x-mpegurl",                   ".m3u"},
      {"audio/x-mpg",                       ".mp3"},
      {"audio/x-ms-wma",                    ".wma"},
      {"audio/x-wav",                       ".wav"},
      //{"audio/", "."},

      // video
      {"video/avi",                         ".avi"},
      {"video/mpeg",                        ".mpeg"},
      {"video/mp4",                         ".mp4"},
      {"video/mpg",                         ".mpg"},
      {"video/msvideo",                     ".avi"},
      {"video/webm",                        ".webm"},
      {"video/x-flv",                       ".flv"},
      {"video/x-ms-wmv",                    ".wmv"},
      {"video/x-msvideo",                   ".avi"},
      //{"video/", "."},

      // application
      {"application/atom+xml",              ".xml"},
      {"application/excel",                 ".xls"},
      {"application/gzip",                  ".gz"},
      {"application/javascript",            ".js"},
      {"application/lha",                   ".lzh"},
      {"application/msexcel",               ".xls"},
      {"application/mspowerpoint",          ".ppt"},
      {"application/msword",                ".doc"},
      {"application/octet-stream",          string.Empty},
      {"application/ogg",                   ".ogg"},
      {"application/pdf",                   ".pdf"},
      {"application/pot",                   ".pot"},
      {"application/powerpoint",            ".ppt"},
      {"application/pps",                   ".pps"},
      {"application/ppt",                   ".ppt"},
      {"application/rdf+xml",               ".rdf"},
      {"application/rtf",                   ".rtf"},
      {"application/rss+xml",               ".xml"},
      {"application/vnd.ms-excel",          ".xls"},
      {"application/vnd.ms-powerpoint",     ".ppt"},
      {"application/x-tar",                 ".tar"},
      {"application/x-exe",                 ".exe"},
      {"application/x-gzip",                ".gz"},
      {"application/x-lha",                 ".lzh"},
      {"application/x-lha-compressed",      ".lzh"},
      {"application/x-lhz",                 ".lzh"},
      {"application/x-msexcel",             ".xls"},
      {"application/x-zip-compressed",      ".zip"},
      {"application/xhtml+xml",             ".xhtml"},
      {"application/zip",                   ".zip"},
      //{"application/", "."},
    };
  }
}

