using System;
using System.IO;
using System.Linq;
using NUnit.Framework;

namespace Smdn.Formats.Mime {
  [TestFixture]
  public class MailBaseTests {
    private class MailImpl : MailBase, IDisposable {
      public MailImpl(MimeMessage baseMessage)
        : base(baseMessage)
      {
      }

      public void Dispose()
      {
        base.DisposeCore();
      }
    }

    private static MailBase CreateMail(string input)
    {
      return new MailImpl(MimeMessage.LoadMessage(input));
    }

    [Test]
    public void TestDispose()
    {
      var input = @"MIME-Version: 1.0";

      var mail = CreateMail(input);

      (mail as IDisposable).Dispose();

      Assert.Throws<ObjectDisposedException>(() => Assert.IsNotNull(mail.BaseMessage));

      Assert.Throws<ObjectDisposedException>(() => mail.ReadContent((Action<Stream>)((s) => {})));
      Assert.Throws<ObjectDisposedException>(() => mail.ReadContent((Action<StreamReader>)((r) => {})));
      Assert.Throws<ObjectDisposedException>(() => mail.ReadContent((Action<BinaryReader>)((r) => {})));
      Assert.Throws<ObjectDisposedException>(() => mail.ReadContentAsText());
      Assert.Throws<ObjectDisposedException>(() => mail.ReadContentAsBinary());
      Assert.Throws<ObjectDisposedException>(() => mail.CopyContentTo(Stream.Null));
      Assert.Throws<ObjectDisposedException>(() => mail.CopyContentTo(Stream.Null, true));
      Assert.Throws<ObjectDisposedException>(() => mail.SaveContent(string.Empty));
      Assert.Throws<ObjectDisposedException>(() => mail.SaveContent(string.Empty, true));
      Assert.Throws<ObjectDisposedException>(() => mail.ToString());

      Assert.DoesNotThrow(() => (mail as IDisposable).Dispose(), "re-Dispose");
    }

    [Test]
    public void TestToString()
    {
      var input = @"Content-Type: text/plain

body".Replace("\r\n", "\n").Replace("\n", "\r\n");

      var mail = CreateMail(input);

      Assert.AreEqual(input, mail.ToString());
    }
  }
}

